# Rare earth mineral diversity controlled by REE pattern shapes
# Michael Anenburg
# Mineralogical Magazine

# color palette, see vector col.pal below
# library(RColorBrewer)

# REE PATTERNS ------------------------------------------------------------

# resolution reduction factor, 1 = highest resolution, higher numbers lead to
# quicker calculations. 4 and lower is good enough.
red.fact <- 1

fig.size <- 18.1
lt.cex <- 0.9
mar.line <- "grey"
lty.grid <- "solid"
col.grid <- "grey97"

ltrs <- c(letters, "alpha", "beta", "gamma", "delta")

# definitions -------------------------------------------------------------

#https://iupac.org/wp-content/uploads/2018/12/IUPAC_Periodic_Table-01Dec18.jpg
ms.O <- 15.999
mass <- c(138.91, # La
          140.12, # Ce
          140.91, # Pr
          144.24, # Nd
          150.36, # Sm
          151.96, # Eu
          157.25, # Gd
          158.93, # Tb
          162.50, # Dy
          164.93, # Ho
          167.26, # Er
          168.93, # Tm
          173.05, # Yb
          174.97) # Lu

# From O'Neill 2016
radii <- c(1.160, # La
           1.143, # Ce
           1.126, # Pr
           1.109, # Nd
           1.079, # Sm
           1.066, # Eu
           1.053, # Gd
           1.040, # Tb
           1.027, # Dy
           1.015, # Ho
           1.004, # Er
           0.994, # Tm
           0.985, # Yb
           0.977) # Lu

# From O'Neill 2016
CI <- c(0.2472, # La
        0.6308, # Ce
        0.0950, # Pr
        0.4793, # Nd
        0.1542, # Sm
        0.0592, # Eu
        0.2059, # Gd
        0.0375, # Tb
        0.2540, # Dy
        0.0554, # Ho
        0.1645, # Er
        0.0258, # Tm
        0.1684, # Yb
        0.0251) # Lu

names(radii) <- names(CI) <- c("La",
                               "Ce",
                               "Pr",
                               "Nd",
                               "Sm",
                               "Eu",
                               "Gd",
                               "Tb",
                               "Dy",
                               "Ho",
                               "Er",
                               "Tm",
                               "Yb",
                               "Lu")

# From O'Neill 2016
f.matrix <- matrix(data = c(+0.1052, +4.913, +0.243, +8.29, # La
                            +0.0882, +2.033, -0.031, -7.16, # Ce
                            +0.0712, -0.270, -0.172, -7.98, # Pr
                            +0.0542, -1.994, -0.208, -1.99, # Nd
                            +0.0242, -3.627, -0.108, +8.64, # Sm
                            +0.0112, -3.776, -0.032, +9.67, # Eu
                            -0.0018, -3.587, +0.043, +7.91, # Gd
                            -0.0148, -3.059, +0.105, +3.82, # Tb
                            -0.0278, -2.194, +0.142, -1.45, # Dy
                            -0.0398, -1.095, +0.140, -5.80, # Ho
                            -0.0508, +0.165, +0.101, -7.63, # Er
                            -0.0608, +1.521, +0.026, -5.90, # Tm
                            -0.0698, +2.912, -0.078, -0.16, # Yb
                            -0.0778, +4.284, -0.203, +9.41),# Lu
                   ncol = 4, byrow = TRUE)


# functions ---------------------------------------------------------------

lambda2ratio <- function(l0 = 1,
                         l1 = 0,
                         l2 = 0,
                         l3 = 0,
                         l4 = 0,
                         Ce = 1,
                         Eu = 1,
                         YHo = 25){
  # Multiply the given lambda values with the matrix
  calc.mat <- apply(t(t(f.matrix) * c(l1 / 1E0,
                                      l2 / 1E3,
                                      l3 / 1E3,
                                      l4 / 1E6)),
                    MARGIN = 1, FUN = sum)
  # Add lambda0, does not need matrix math
  mat.l0 <- calc.mat + l0
  # Results in ln, convert to log10
  log10.corr <- mat.l0 * log10(exp(1))
  # Function returns the CI-normalised mass of each RE
  return(10^log10.corr)
}

# calculation input -------------------------------------------------------

# ranges of the lambda values
l1.seq <- c(-25, -17, -12, -9, -7, -5, -3, 0, 5, 15)
l2.seq <- seq(-750, 750, 4 * red.fact)
l3.seq <- seq(-6500, 6500, 40 * red.fact)
l4.seq <- c(-40000, 0, 40000)

# calculation ------------------------------------------------------------

# The data will be a list, with each list item corresponding to a different l4
df.list <- vector("list", length(l4.seq))

# File name to save the data to hard drive
fn <- "patterndata"

# Structure for the following data.frames
# l0 is not important for this study, given arbitrary value of 7
df.str <- data.frame(l1 = NA,
                     l2 = NA,
                     l3 = NA,
                     ree.max = NA, ree.max2 = NA,
                     eu.ratio = NA, eu.ratio2 = NA)

# No need to rerun calculations if the file exists. If there are any changes,
# manually erase the file first so it rebuilds the data.
if (file.exists(fn)){
  load(fn)
} else
{
  for (l4i in l4.seq) {
    # Just a way to convert the three lambda4 values to 1:3
    idx <- l4i/40000+2
    
    # Initialise data.frame
    df <- df.str
    
    # Using intermediate data.frames for increased performance via df.str
    for (i in l1.seq) {
      df.j <- df.str
      for(j in l2.seq){
        df.k <- df.str
        for (k in l3.seq){
          # generate a pattern
          ree.pat <- lambda2ratio(7, i, j, k, l4i,
                                  Ce = 1, Eu = 1)
          # calculate the ppm values
          ree.ppm <- ree.pat * CI
          # add Y with Y/Ho = 25.5
          ree.ppm <- c(ree.ppm, ree.ppm[10] * 25.5)
          # calculate atomic values, incl Y mass
          ree.mol <- ree.ppm / c(mass, 88.90584)
          # find the maximum
          ree.max <- which.max(ree.mol)
          # zero Ce and Y
          ree.mol2 <- ree.mol
          ree.mol2[c(2,15)] <- 0
          ree.max2 <- which.max(ree.mol2)
          # Eu anomaly needed to make Eu mineral
          eu.ratio <- ree.mol[ree.max] / ree.mol[6]
          eu.ratio2 <- ree.mol2[ree.max2] / ree.mol2[6]
          
          df.k <- rbind(df.k, c(i, j, k,
                                ree.max, ree.max2,
                                eu.ratio, eu.ratio2))
        }
        print(j)
        # [-1,] to remove the first NA value formed using data.frame()
        df.j <- rbind(df.j, df.k[-1, ])
      }
      print(i)
      df <- rbind(df, df.j[-1, ])
    }
    df.list[[idx]] <- df[-1, ]
  }
  save(df.list, file = fn)
}


# figure 1 - lambdas and ratios -------------------------------------------

plotaxis1 <- function() {
  axis(1, at = radii * 100, labels = NA,
       lwd = 0, lwd.ticks = 1,
       tcl = 0.25, mgp = c(-3, -1.5, 0))
  text(x = radii * 100, y = ylow * 0.9, labels = names(radii),
       cex = 0.8, font = 1, srt = 90, adj = 0)
  axis(1, tcl = -0.25, mgp = c(2, 0.25, 0), lwd = 0, lwd.ticks = 1)
}

pdf("1lambdasratios.pdf", width = fig.size / 2.54, height = 9 / 2.54,
    pointsize = 10, family = "Bookman")

par(mfrow = c(1, 3), cex = 1, oma = c(1.1, 2.2, 0, 0))

ylow <- 4
yhigh <- 500

# first

par(mar = c(1, 0, 0.05, 0))

plot(x = 1, y = 1, type = "n", xlim = c(range(radii)[2], range(radii)[1]) * 100,
     bty = "n", log = "y", ylim = c(ylow, yhigh),
     xaxt = "n", yaxt = "n",
     xlab = "", ylab = "")

abline(v = radii * 100, col = col.grid, lty = lty.grid)
abline(h = seq(4, 9, 1), col = col.grid)
abline(h = seq(10, 90, 10), col = col.grid)
abline(h = seq(100, 600, 100), col = col.grid)

ree.pat <- lambda2ratio(l0 = 5, l1 = -20, l2 = -200, l3 = -1000, l4 = 10000)
ree.pat[6] <- ree.pat[6] * 0.2

lines(x = radii * 100, y = ree.pat)

axis(2, lwd = 0, lwd.ticks = 1,
     labels = c(expression(10^-1),
                expression(10^0),
                expression(10^1),
                expression(10^2),
                expression(10^3)),
     at = c(0.1, 1, 10, 100, 1000),
     mgp = c(1, 0.25, 0), tcl = -0.25)

box()

text(x = 115.5, y = 510, labels = "a.")

xpos <- 99
ypos <- -6
mtext(expression(lambda[0] == 5), side = 3, line = ypos, at = xpos, adj = 1, cex = 0.8)
mtext(expression(lambda[1] == -20), side = 3, line = ypos - 1, at = xpos, adj = 1, cex = 0.8)
mtext(expression(lambda[2] == -200), side = 3, line = ypos - 2, at = xpos, adj = 1, cex = 0.8)
mtext(expression(lambda[3] == -1000), side = 3, line = ypos - 3, at = xpos, adj = 1, cex = 0.8)
mtext(expression(lambda[4] == 10000), side = 3, line = ypos - 4, at = xpos, adj = 1, cex = 0.8)

mtext(bquote(La[N]/Lu[N] == .(signif(ree.pat[1] / ree.pat[14], 2))),
      side = 3, line = ypos - 7, at = xpos, adj = 1, cex = 0.8)
mtext(bquote(La[N]/Sm[N] == .(signif(ree.pat[1] / ree.pat[5], 2))),
      side = 3, line = ypos - 8, at = xpos, adj = 1, cex = 0.8)
mtext(bquote(La[N]/Gd[N] == .(signif(ree.pat[1] / ree.pat[7], 2))),
      side = 3, line = ypos - 9, at = xpos, adj = 1, cex = 0.8)
mtext(bquote(Sm[N]/Lu[N] == .(signif(ree.pat[5] / ree.pat[14], 2))),
      side = 3, line = ypos - 10, at = xpos, adj = 1, cex = 0.8)
mtext(bquote(Gd[N]/Lu[N] == .(signif(ree.pat[7] / ree.pat[14], 2))),
      side = 3, line = ypos - 11, at = xpos, adj = 1, cex = 0.8)

plotaxis1()

# second

par(mar = c(1, 0, 0.05, 0))

plot(x = 1, y = 1, type = "n", xlim = c(range(radii)[2], range(radii)[1]) * 100,
     bty = "n", log = "y", ylim = c(ylow, yhigh),
     xaxt = "n", yaxt = "n",
     xlab = "", ylab = "")

abline(v = radii * 100, col = col.grid, lty = lty.grid)
abline(h = seq(0.07, 0.09, 0.01), col = col.grid)
abline(h = seq(0.1, 0.9, 0.1), col = col.grid)
abline(h = seq(1, 9, 1), col = col.grid)
abline(h = seq(10, 90, 10), col = col.grid)
abline(h = seq(100, 900, 100), col = col.grid)
abline(h = seq(1000, 2000, 1000), col = col.grid)

abline(v = radii * 100, col = col.grid, lty = lty.grid)

ree.pat <- lambda2ratio(l0 = 4.5, l1 = 15, l2 = -200, l3 = -1000, l4 = 10000)
ree.pat[6] <- ree.pat[6] * 0.2

lines(x = radii * 100, y = ree.pat)

box()

text(x = 115.5, y = 510, labels = "b.")

xpos <- 114.5
ypos <- -6
mtext(expression(lambda[0] == 5), side = 3, line = ypos, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[1] == 15), side = 3, line = ypos - 1, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[2] == -200), side = 3, line = ypos - 2, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[3] == -1000), side = 3, line = ypos - 3, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[4] == 10000), side = 3, line = ypos - 4, at = xpos, adj = 0, cex = 0.8)

mtext(bquote(La[N]/Lu[N] == .(signif(ree.pat[1] / ree.pat[14], 2))),
      side = 3, line = ypos - 7, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(La[N]/Sm[N] == .(signif(ree.pat[1] / ree.pat[5], 2))),
      side = 3, line = ypos - 8, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(La[N]/Gd[N] == .(signif(ree.pat[1] / ree.pat[7], 2))),
      side = 3, line = ypos - 9, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(Sm[N]/Lu[N] == .(signif(ree.pat[5] / ree.pat[14], 2))),
      side = 3, line = ypos - 10, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(Gd[N]/Lu[N] == .(signif(ree.pat[7] / ree.pat[14], 2))),
      side = 3, line = ypos - 11, at = xpos, adj = 0, cex = 0.8)

plotaxis1()

# third

par(mar = c(1, 0, 0.05, 0.1))

plot(x = 1, y = 1, type = "n", xlim = c(range(radii)[2], range(radii)[1]) * 100,
     bty = "n", log = "y", ylim = c(ylow, yhigh),
     xaxt = "n", yaxt = "n",
     xlab = "", ylab = "")

abline(v = radii * 100, col = col.grid, lty = lty.grid)
abline(h = seq(0.07, 0.09, 0.01), col = col.grid)
abline(h = seq(0.1, 0.9, 0.1), col = col.grid)
abline(h = seq(1, 9, 1), col = col.grid)
abline(h = seq(10, 90, 10), col = col.grid)
abline(h = seq(100, 900, 100), col = col.grid)
abline(h = seq(1000, 2000, 1000), col = col.grid)

abline(v = radii * 100, col = col.grid, lty = lty.grid)

ree.pat <- lambda2ratio(l0 = 4.5, l1 = 15, l2 = 0, l3 = -1000, l4 = 10000)
ree.pat[6] <- ree.pat[6] * 0.2

lines(x = radii * 100, y = ree.pat)

plotaxis1()
box()

text(x = 115.5, y = 510, labels = "c.")

xpos <- 115
ypos <- -6
mtext(expression(lambda[0] == 5), side = 3, line = ypos, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[1] == 15), side = 3, line = ypos - 1, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[2] == 0), side = 3, line = ypos - 2, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[3] == -1000), side = 3, line = ypos - 3, at = xpos, adj = 0, cex = 0.8)
mtext(expression(lambda[4] == 10000), side = 3, line = ypos - 4, at = xpos, adj = 0, cex = 0.8)

mtext(bquote(La[N]/Lu[N] == .(signif(ree.pat[1] / ree.pat[14], 2))),
      side = 3, line = ypos - 7, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(La[N]/Sm[N] == .(signif(ree.pat[1] / ree.pat[5], 2))),
      side = 3, line = ypos - 8, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(La[N]/Gd[N] == .(signif(ree.pat[1] / ree.pat[7], 2))),
      side = 3, line = ypos - 9, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(Sm[N]/Lu[N] == .(signif(ree.pat[5] / ree.pat[14], 2))),
      side = 3, line = ypos - 10, at = xpos, adj = 0, cex = 0.8)
mtext(bquote(Gd[N]/Lu[N] == .(signif(ree.pat[7] / ree.pat[14], 2))),
      side = 3, line = ypos - 11, at = xpos, adj = 0, cex = 0.8)

mtext("Sample / Chondrite", side = 2, outer = TRUE, line = 1.2)
mtext("Radius (pm)", side = 1, outer = TRUE)

dev.off()

# figure 2 - examples -----------------------------------------------------

# height adjusted to make all panels square(ish)
pdf("2patterns.pdf", width = fig.size / 2.54, height = fig.size / 2.54 / 5 * 2 * 3,
    pointsize = 10, family = "Bookman")
# Lambda is serif, and Bookman is more readable than default serif Times

par(mfrow = c(6,5),
    mar = c(0, 0, 1, 0),
    oma = c(1, 1, 0, 1),
    cex = 1, xpd = TRUE)

ltrs.idx <- 1

# dfl is the list index, i.e. lambda4
for (dfl in 1:3) {
  
  for (i in unique(df.list[[dfl]]$l1)){
    # make a blank plot
    plot(1, 1, xaxt = "n", yaxt = "n", type = "n",
         xlim = c(min(l2.seq), max(l2.seq)),
         ylim = c(min(l3.seq), max(l3.seq)))
    # lambda1 label on top of each panel
    mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
    segments(-700, 7400, -360, 8000, col = mar.line)
    segments(700, 7400, 360, 8000, col = mar.line)
    if (ltrs.idx < 27) {
      mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
    }
    if (ltrs.idx == 27) {
      mtext(text = expression(alpha), side = 3, line = 0.1, adj = 0.01, cex = 0.8)
    }
    if (ltrs.idx == 28) {
      mtext(text = expression(beta), side = 3, line = -0.1, adj = 0.01, cex = 0.8)
    }
    if (ltrs.idx == 29) {
      mtext(text = expression(gamma), side = 3, line = 0, adj = 0.01, cex = 0.8)
    }
    if (ltrs.idx == 30) {
      mtext(text = expression(delta), side = 3, line = 0, adj = 0.01, cex = 0.8)
    }
    ltrs.idx <- ltrs.idx + 1
    
    # make all of the cute REE plots
    for (j in seq(-650, 650, 216 + 2/3)) {
      for (k in seq(-5500, 5500, 1833 + 1/3)) {
        # generate the pattern
        small.plot <- log10(lambda2ratio(l1 = i, l2 = j, l3 = k, l4 = l4.seq[dfl]))
        # normalise it so it's centred
        xnorm <- 0.5 - (radii - min(radii)) / (max(radii) - min(radii))
        ynorm <- small.plot - (max(small.plot) + min(small.plot))/2
        # shift and enlarge
        lines(x = xnorm * 190 + j, y = ynorm * 700 + k)
      }}
  }
}

# labels on the margins
mtext(expression(lambda[4] == -40000), side = 4, outer = TRUE, line = 0.1, adj = 0.86)
mtext(expression(lambda[4] == 0), side = 4, outer = TRUE, line = 0.1, adj = 0.49)
mtext(expression(lambda[4] == 40000), side = 4, outer = TRUE, line = 0.1, adj = 0.12)
mtext(expression(lambda[2] == "[-750, 750]"), side = 1, outer = TRUE, line = 0.1)
mtext(expression(lambda[3] == "[-6500, 6500]"), side = 2, outer = TRUE, line = 0)

par(new = TRUE, mfrow = c(1, 1), mar = c(0, 0, 0, 0), oma = c(0, 0, 0, 0))
plot(1, type = "n", axes = F, ann = F, xaxs = "i", yaxs = "i", 
     xlim = c(0, 1), ylim = c(0, 1))

segments(0.02, 0.03, 0.01, 0.42, col = mar.line) # left bottom
segments(0.02, 0.97, 0.01, 0.6, col = mar.line) # left top
segments(0.027, 0.016, 0.42, 0.008, col = mar.line) # bottom left
segments(0.971, 0.016, 0.58, 0.008, col = mar.line) # bottom right

segments(0.98, 0.03, 0.99, 0.12, col = mar.line) # right bottom
segments(0.98, 0.32, 0.99, 0.23, col = mar.line)
segments(0.98, 0.36, 0.99, 0.46, col = mar.line) # right mid
segments(0.98, 0.65, 0.99, 0.54, col = mar.line)
segments(0.98, 0.97, 0.99, 0.88, col = mar.line) # right top
segments(0.98, 0.68, 0.99, 0.77, col = mar.line)

dev.off()


# figure 3 - l4=0, with and without Ce and Y ------------------------------

pdf("3resultl4zero.pdf", width = fig.size / 2.54, height = fig.size / 2.54 / 5 * 2 * 2,
    pointsize = 10, family = "Bookman")


par(mfrow = c(4,5),
    mar = c(0, 0, 1, 0),
    oma = c(1, 1, 0.6, 1),
    cex = 1, xpd = TRUE)

ltrs.idx <- 1

for (i in unique(df.list[[dfl]]$l1)){
  df.s <- df.list[[2]][df.list[[2]]$l1 == i, ]
  df.m <- xtabs(ree.max ~ l2 + l3, data = df.s)
  image(df.m, col = hcl.colors(15, palette = "Plasma"), zlim = c(1, 15),
        xaxt = "n", yaxt = "n", useRaster = TRUE, bty = "n")
  box()
  mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
  segments(0.05, 1.02, 0.29, 1.07, col = mar.line)
  segments(0.71, 1.07, 0.95, 1.02, col = mar.line)
  mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
  ltrs.idx <- ltrs.idx + 1
  }

for (i in unique(df.list[[dfl]]$l1)){
  df.s <- df.list[[2]][df.list[[2]]$l1 == i, ]
  df.m <- xtabs(ree.max2 ~ l2 + l3, data = df.s)
  image(df.m, col = hcl.colors(15, palette = "Plasma"), zlim = c(1, 15),
        xaxt = "n", yaxt = "n", useRaster = TRUE, bty = "n")
  box()
  mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
  segments(0.05, 1.02, 0.29, 1.07, col = mar.line)
  segments(0.71, 1.07, 0.95, 1.02, col = mar.line)
  mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
  ltrs.idx <- ltrs.idx + 1
}

mtext("with Ce & Y", side = 4, outer = TRUE, line = 0, adj = 0.77)
mtext("without Ce & Y", side = 4, outer = TRUE, line = 0, adj = 0.18)
mtext(expression(lambda[2] == "[-750, 750]"), side = 1, outer = TRUE, line = 0.1)
mtext(expression(lambda[3] == "[-6500, 6500]"), side = 2, outer = TRUE, line = 0)
mtext(expression(lambda[4] == 0), side = 3, outer = TRUE, line = -0.5, adj = -0.02)

par(mfg = c(1,1))
text(0.85, 0.15, "Lu", cex = lt.cex)
segments(0.9, 0.1, 0.98 , 0.02)

par(mfg = c(1,3))
text(0.2, 0.7, "Y", cex = lt.cex)
text(0.9, 0.2, "Yb", cex = lt.cex)

par(mfg = c(2,4))
text(0.2, 0.1, "Nd", cex = lt.cex, col = "white")
text(0.7, 0.2, "Ce", cex = lt.cex, col = "white")
text(0.8, 0.8, "La", cex = lt.cex, col = "white")

par(mfg = c(3,1))
text(0.6, 0.6, "Er", cex = lt.cex)
segments(0.6, 0.65, 0.57 , 0.9)

par(mfg = c(3,2))
text(0.1, 0.35, "Sm", cex = lt.cex, col = "white")
segments(0.1, 0.29, 0.02, 0.16, col = "white")

par(mfg = c(3,3))
text(0.2, 0.1, "Nd", cex = lt.cex, col = "white")
text(0.7, 0.3, "Yb", cex = lt.cex)
text(0.8, 0.8, "La", cex = lt.cex, col = "white")
text(0.1, 0.5, "Gd", cex = lt.cex, col = "white")
text(0.35, 0.8, "Dy", cex = lt.cex, col = "white")

par(mfg = c(4,5))
text(0.4, 0.2, "Nd", cex = lt.cex, col = "white")
text(0.7, 0.7, "La", cex = lt.cex, col = "white")

par(new = TRUE, mfrow = c(1, 1), mar = c(0, 0, 0, 0), oma = c(0, 0, 1, 0))
plot(1, type = "n", axes = F, ann = F, xaxs = "i", yaxs = "i", 
     xlim = c(0, 1), ylim = c(0, 1))

segments(0.02, 0.05, 0.01, 0.4, col = mar.line) # left bottom
segments(0.02, 0.95, 0.01, 0.64, col = mar.line) # left top
segments(0.027, 0.02, 0.42, 0.01, col = mar.line) # bottom left
segments(0.971, 0.02, 0.58, 0.01, col = mar.line) # bottom right

segments(0.98, 0.05, 0.99, 0.16, col = mar.line) # right bottom
segments(0.98, 0.47, 0.99, 0.36, col = mar.line)
segments(0.98, 0.97, 0.99, 0.84, col = mar.line) # right top
segments(0.98, 0.53, 0.99, 0.66, col = mar.line)

dev.off()


# figure 4 - l4=+-40000 ---------------------------------------------------

pdf("4resultl4posneg.pdf", width = fig.size / 2.54, height = fig.size / 2.54 / 5 * 2 * 2,
    pointsize = 10, family = "Bookman")


par(mfrow = c(4,5),
    mar = c(0, 0, 1, 0),
    oma = c(1, 1, 0, 1),
    cex = 1, xpd = TRUE)

ltrs.idx <- 1

for (i in unique(df.list[[dfl]]$l1)){
  df.s <- df.list[[1]][df.list[[2]]$l1 == i, ]
  df.m <- xtabs(ree.max2 ~ l2 + l3, data = df.s)
  image(df.m, col = hcl.colors(15, palette = "Plasma"), zlim = c(1, 15),
        xaxt = "n", yaxt = "n", useRaster = TRUE, bty = "n")
  box()
  mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
  segments(0.05, 1.02, 0.29, 1.07, col = mar.line)
  segments(0.71, 1.07, 0.95, 1.02, col = mar.line)
  mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
  ltrs.idx <- ltrs.idx + 1
}

for (i in unique(df.list[[dfl]]$l1)){
  df.s <- df.list[[3]][df.list[[2]]$l1 == i, ]
  df.m <- xtabs(ree.max2 ~ l2 + l3, data = df.s)
  image(df.m, col = hcl.colors(15, palette = "Plasma"), zlim = c(1, 15),
        xaxt = "n", yaxt = "n", useRaster = TRUE, bty = "n")
  box()
  mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
  segments(0.05, 1.02, 0.29, 1.07, col = mar.line)
  segments(0.71, 1.07, 0.95, 1.02, col = mar.line)
  mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
  ltrs.idx <- ltrs.idx + 1
}

mtext(expression(lambda[2] == "[-750, 750]"), side = 1, outer = TRUE, line = 0.1)
mtext(expression(lambda[3] == "[-6500, 6500]"), side = 2, outer = TRUE, line = 0)
mtext(expression(lambda[4] == -40000), side = 4, outer = TRUE, line = 0.1, adj = 0.78)
mtext(expression(lambda[4] == 40000), side = 4, outer = TRUE, line = 0.1, adj = 0.2)

par(mfg = c(1,1))
text(0.57, 0.9, "Er", cex = lt.cex)

par(mfg = c(1,3))
text(0.75, 0.3, "Yb", cex = lt.cex)

par(mfg = c(2,2))
text(0.3, 0.8, "Dy", cex = lt.cex, col = "white")

par(mfg = c(2,3))
text(0.15, 0.4, "Gd", cex = lt.cex, col = "white")
segments(0.15, 0.47, 0.06, 0.67, col = "white")

par(mfg = c(2,5))
text(0.3, 0.3, "Nd", cex = lt.cex, col = "white")
text(0.8, 0.8, "La", cex = lt.cex, col = "white")

par(mfg = c(3,2))
text(0.15, 0.35, "Sm", cex = lt.cex, col = "white")
segments(0.15, 0.29, 0.08, 0.09, col = "white")

par(mfg = c(3,3))
text(0.35, 0.1, "Nd", cex = lt.cex, col = "white")
text(0.2, 0.5, "Gd", cex = lt.cex, col = "white")
text(0.42, 0.8, "Dy", cex = lt.cex, col = "white")

par(mfg = c(3,4))
text(0.7, 0.3, "Yb", cex = lt.cex)
text(0.8, 0.8, "La", cex = lt.cex, col = "white")

par(mfg = c(3,5))
text(0.8, 0.2, "Lu", cex = lt.cex)
segments(0.85, 0.15, 0.95 , 0.05)

par(mfg = c(4,4))
text(0.15, 0.15, "Sm", cex = lt.cex, col = "white")
segments(0.15, 0.21, 0.05, 0.4, col = "white")

par(new = TRUE, mfrow = c(1, 1), mar = c(0, 0, 0, 0), oma = c(0, 0, 0, 0))
plot(1, type = "n", axes = F, ann = F, xaxs = "i", yaxs = "i", 
     xlim = c(0, 1), ylim = c(0, 1))

segments(0.02, 0.05, 0.01, 0.4, col = mar.line) # left bottom
segments(0.02, 0.95, 0.01, 0.64, col = mar.line) # left top
segments(0.027, 0.022, 0.42, 0.01, col = mar.line) # bottom left
segments(0.971, 0.022, 0.58, 0.01, col = mar.line) # bottom right

segments(0.98, 0.05, 0.99, 0.18, col = mar.line) # right bottom
segments(0.98, 0.46, 0.99, 0.33, col = mar.line)
segments(0.98, 0.97, 0.99, 0.84, col = mar.line) # right top
segments(0.98, 0.53, 0.99, 0.66, col = mar.line)

dev.off()


# figure 5 - Eu anomaly example -------------------------------------------

pdf("5euanomaly.pdf", width = 8.8 / 2.54, height = 9 / 2.54,
    pointsize = 10, family = "Bookman")

par(mar = c(2.4, 1.9, 0.1, 0.1),
    tcl = 0.25, mgp = c(1, 0, 0))

anomal.pat <- lambda2ratio(l0 = 1, l2 = -300, l3 = -1000)

plot(x = radii, y = anomal.pat, type = "n", xlim = c(111, 103.5),
     bty = "n", log = "y", ylim = c(5.05, 9),
     xaxt = "n", yaxt = "n",
     xlab = "Radius (pm)", ylab = "Sample / Chondrite")

abline(h = seq(5, 9, 1), col = col.grid, lty = lty.grid)

segments(x0 = radii[6] * 100, y0 = anomal.pat[6] * 0.7, y1 = anomal.pat[6], lty = "dashed")
points(x = rep(radii[6] * 100, 2), y = c(anomal.pat[6], sqrt(anomal.pat[5] * anomal.pat[7])),
       col = hcl.colors(3, palette = "Dark 2")[2:3])

lines(x = radii[c(5,7)] * 100, y = anomal.pat[c(5,7)], col = hcl.colors(3, palette = "Dark 2")[3])
lines(x = radii[5:7] * 100, y = anomal.pat[5:7], col = hcl.colors(3, palette = "Dark 2")[2])


anomal.pat[6] <- anomal.pat[6] * 0.7
lines(x = radii * 100, y = anomal.pat)

points(x = radii * 100, y = anomal.pat)

axis(1, at = radii * 100, labels = names(radii),
     lwd = 0, lwd.ticks = 1,
     tcl = 0.25, mgp = c(-3, -1.5, 0))
axis(1, tcl = -0.25, mgp = c(2, 0.25, 0), lwd = 0, lwd.ticks = 1)
axis(2, lwd = 0, lwd.ticks = 1)
box()

mtext(expression(lambda[0] == 1 ~~~~~ lambda[1] == 0 ~~~~~ lambda[2] == -300 ~~~~~ lambda[3] == -1000), side = 3, outer = TRUE, line = -0.1)

dev.off()

# figure 6 - Eu ratios ----------------------------------------------------

pdf("6contours.pdf", width = 19 / 2.54, height = 19 / 2.54 / 5 * 2 * 2,
    pointsize = 10, family = "Bookman")

par(mfrow = c(4,5),
    mar = c(0, 0, 1, 0),
    oma = c(1, 1, 0.6, 1),
    cex = 1, xpd = TRUE)

con.levels <- c(2, 3, 4, 5, 10, 15, 25, 50, 100, 200, 400, 1000, 2000, 4000, 10000, 20000, 40000)

ltrs.idx <- 1

for (i in unique(df.list[[dfl]]$l1)){
  df.s <- df.list[[2]][df.list[[2]]$l1 == i, ]
  df.m <- xtabs(eu.ratio ~ l2 + l3, data = df.s)
  contour(df.m, levels = con.levels,
          xaxt = "n", yaxt = "n", xaxs = "i", yaxs = "i")
  mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
  segments(0.05, 1.02, 0.29, 1.07, col = mar.line)
  segments(0.71, 1.07, 0.95, 1.02, col = mar.line)
  mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
  ltrs.idx <- ltrs.idx + 1
}

for (i in unique(df.list[[dfl]]$l1)){
  df.s <- df.list[[2]][df.list[[2]]$l1 == i, ]
  df.m <- xtabs(eu.ratio2 ~ l2 + l3, data = df.s)
  contour(df.m, levels = con.levels,
          xaxt = "n", yaxt = "n", xaxs = "i", yaxs = "i")
  mtext(text = bquote(lambda[1] == .(i)), side = 3, line = -0.1)
  segments(0.05, 1.02, 0.29, 1.07, col = mar.line)
  segments(0.71, 1.07, 0.95, 1.02, col = mar.line)
  mtext(text = ltrs[ltrs.idx], side = 3, line = 0.1, adj = 0.01, cex = 0.8)
  ltrs.idx <- ltrs.idx + 1
}
mtext("with Ce & Y", side = 4, outer = TRUE, line = 0, adj = 0.77)
mtext("without Ce & Y", side = 4, outer = TRUE, line = 0, adj = 0.18)
mtext(expression(lambda[2] == "[-750, 750]"), side = 1, outer = TRUE, line = 0.1)
mtext(expression(lambda[3] == "[-6500, 6500]"), side = 2, outer = TRUE, line = 0)
mtext(expression(lambda[4] == 0), side = 3, outer = TRUE, line = -0.5, adj = -0.02)

par(new = TRUE, mfrow = c(1, 1), mar = c(0, 0, 0, 0), oma = c(0, 0, 1, 0))
plot(1, type = "n", axes = F, ann = F, xaxs = "i", yaxs = "i", 
     xlim = c(0, 1), ylim = c(0, 1))

segments(0.02, 0.05, 0.01, 0.4, col = mar.line) # left bottom
segments(0.02, 0.95, 0.01, 0.64, col = mar.line) # left top
segments(0.027, 0.022, 0.42, 0.01, col = mar.line) # bottom left
segments(0.971, 0.022, 0.58, 0.01, col = mar.line) # bottom right

segments(0.98, 0.05, 0.99, 0.16, col = mar.line) # right bottom
segments(0.98, 0.47, 0.99, 0.36, col = mar.line)
segments(0.98, 0.97, 0.99, 0.84, col = mar.line) # right top
segments(0.98, 0.53, 0.99, 0.66, col = mar.line)

dev.off()

# figure 7 - Comparison with natural minerals  -------------------------------

minerals <- structure(list(Mineral = c("monazite-(Sm)", "florencite-(Sm)", 
                                       "lepersonnite-(Gd)", "samarskite-(Yb)",
                                       "xenotime-(Yb)", "keiviite-(Yb)", "hingganite-(Yb)"),
                           Source = c("Masau2002", "Repina2011", "Deliens1982", 
                                      "Simmons2006", "Buck1999", "Voloshin1983", "Voloshin1984"),
                           Y2O3 = c(1.4, NA, 0.41, 1.92, 8.75, 1.02, 8.56),
                           La2O3 = c(2.02, 0.62, NA, 0.03, NA, NA, NA),
                           Ce2O3 = c(10.07, 3.29, NA, 0.22, NA, NA, NA),
                           Pr2O3 = c(1.6, 1.05, NA, 0.05, NA, NA, NA),
                           Nd2O3 = c(6.28, 0.31, NA, 0.64, NA, NA, NA),
                           Sm2O3 = c(14.29, 12.61, NA, 0.42, NA, NA, NA),
                           Eu2O3 = c(NA, 0.41, NA, NA, NA, NA, NA),
                           Gd2O3 = c(13.48, 2.3, 2.09, 0.55, 0.26, 0.06, 0.45),
                           Tb2O3 = c(1.28, NA, 0.09,0.25, 0.19, 0.04, 0.33),
                           Dy2O3 = c(1.21, 0.4, 1.07, 3.06, 3.72, 1.23, 2.13),
                           Ho2O3 = c(NA, NA, NA, 0.59, 0.94, 0.65, 0.21),
                           Er2O3 = c(NA, NA, NA, 3.4, 8.29, 6.24, 6.47),
                           Tm2O3 = c(NA, NA, NA, 0.58, 2.86, 3.1, 1.38),
                           Yb2O3 = c(0.01, NA, NA, 5.78, 36.37, 55.06, 17.02),
                           Lu2O3 = c(0.05, NA, NA, 0.97, 7.12, 8.97, 1.87)),
                      row.names = c(NA, 7L), class = "data.frame")

min.norm <- apply(minerals[c(1:7), -(1:3)], MARGIN = 1, FUN = function(x) {
  # convert from wt% oxide to ppm
  foo <- x / (mass + ms.O * 1.5) * mass * 10000
  # normalise
  foo <- foo / CI
  foo})

pdf("7comparison.pdf", width = 19 / 2.54, height = 8 / 2.54,
    pointsize = 10, family = "Bookman")

# Based on "Paired" from RColorBrewer package
col.comp <- c("#A6CEE3", "#1F78B4", "#B2DF8A", "#33A02C", "#FB9A99", "#E31A1C", 
              "#FDBF6F", "#FF7F00", "#CAB2D6", "#6A3D9A", "#FFFF99", "#B15928")

col.comp[11] <- "#B19F28" # Because 11 is lemon yellow and light brown is required

par(mfrow = c(1, 3),
    tcl = 0.25, mgp = c(1.1, 0, 0),
    cex = 1, oma = c(1, 0, 0, 0))

# Ytterbium

par(mar = c(1, 2, 0.2, 0))

plot(x = radii, y = anomal.pat, type = "n", xlim = c(range(radii)[2], range(radii)[1]) * 100,
     bty = "n", log = "y", ylim = c(100, 5000000),
     xaxt = "n", yaxt = "n",
     xlab = "", ylab = "Sample / Chondrite")

abline(v = radii * 100, col = col.grid, lty = lty.grid)
abline(h = seq(70, 90, 10), col = col.grid)
abline(h = seq(100, 900, 100), col = col.grid)
abline(h = seq(1000, 9000, 1000), col = col.grid)
abline(h = seq(10000, 90000, 10000), col = col.grid)
abline(h = seq(100000, 900000, 100000), col = col.grid)
abline(h = seq(1000000, 7000000, 1000000), col = col.grid)

# samarskite-(Yb)
points(x = radii * 100, y = min.norm[, 4], col = col.comp[3])
lines(x = radii * 100, y = min.norm[, 4], col = col.comp[3])
lines(x = radii[c(5, 7)] * 100, y = min.norm[c(5, 7), 4], col = col.comp[3], lty = "dashed")
samyb.pat <- lambda2ratio(l0 = 10.49, l1 = -29.56, l2 = -26.4,
                          l3 = -939, l4 = -11996)
lines(x = radii * 100, y = samyb.pat, col = col.comp[4])

# xenotime-(Yb)
points(x = radii * 100, y = min.norm[, 5], col = col.comp[7])
lines(x = radii * 100, y = min.norm[, 5], col = col.comp[7])
xenyb.pat <- lambda2ratio(l0 = 9.21, l1 = -76.26, l2 = 128.3,
                          l3 = 4538, l4 = 43305)
lines(x = radii * 100, y = xenyb.pat, col = col.comp[8])

# keiviite-(Yb)
points(x = radii * 100, y = min.norm[, 6], col = col.comp[9])
lines(x = radii * 100, y = min.norm[, 6], col = col.comp[9])
xenyb.pat <- lambda2ratio(l0 = 9.04, l1 = -72.26, l2 = 277.3,
                          l3 = 367, l4 = -37019)
lines(x = radii * 100, y = xenyb.pat, col = col.comp[10])

# hingganite-(Yb)
points(x = radii * 100, y = min.norm[, 7], col = col.comp[11])
lines(x = radii * 100, y = min.norm[, 7], col = col.comp[11])
hinyb.pat <- lambda2ratio(l0 = 9.86, l1 = -48.11, l2 = 36.6,
                          l3 = 683, l4 = 16993)
lines(x = radii * 100, y = hinyb.pat, col = col.comp[12])

box()

legend(x = 116.5, y = 8000000, title = "samarskite-(Yb)",
       legend = c("data", "fit"),
       col = c(col.comp[3], col.comp[4]),
       lty = "solid", pch = c(1, NA),
       bty = "n", cex = 0.8)

legend(x = 108, y = 8000000, title = "xenotime-(Yb)",
       legend = c("data", "fit"),
       col = c(col.comp[7], col.comp[8]),
       lty = "solid", pch = c(1, NA),
       bty = "n", cex = 0.8)

legend(x = 108, y = 1200000, title = "keiviite-(Yb)",
       legend = c("data", "fit"),
       col = c(col.comp[9], col.comp[10]),
       lty = "solid", pch = c(1, NA),
       bty = "n", cex = 0.8)

legend(x = 116.5, y = 1200000, title = "hingganite-(Yb)",
       legend = c("data", "fit"),
       col = c(col.comp[11], col.comp[12]),
       lty = "solid", pch = c(1, NA),
       bty = "n", cex = 0.8)

text(x = radii * 100, y = 80, labels = names(radii),
     cex = 0.8, font = 1, srt = 90, adj = 0)
axis(1, at = radii * 100, labels = NA,
     lwd = 0, lwd.ticks = 1,
     tcl = 0.25, mgp = c(-3, -1.5, 0),
     cex.axis = 0.5)
axis(1, lwd = 0, lwd.ticks = 1, tcl = -0.25,
     mgp = c(3, 0.3, 0))

axis(2, lwd = 0, lwd.ticks = 1,
     labels = c(expression(10^2),
                expression(10^3),
                expression(10^4),
                expression(10^5),
                expression(10^6)),
     at = c(100, 1000, 10000, 100000, 1000000))

mtext("a.", side = 3, line = -1, adj = -0.1)

# Samarium

par(mar = c(1, 0, 0.2, 2))

plot(x = radii, y = anomal.pat, type = "n", xlim = c(range(radii)[2], range(radii)[1]) * 100,
     bty = "n", log = "y", ylim = c(100, 5000000),
     xaxt = "n", yaxt = "n",
     xlab = "", ylab = "")

abline(v = radii * 100, col = col.grid, lty = lty.grid)
abline(h = seq(70, 90, 10), col = col.grid)
abline(h = seq(100, 900, 100), col = col.grid)
abline(h = seq(1000, 9000, 1000), col = col.grid)
abline(h = seq(10000, 90000, 10000), col = col.grid)
abline(h = seq(100000, 900000, 100000), col = col.grid)
abline(h = seq(1000000, 7000000, 1000000), col = col.grid)

# monazite-(Sm)
points(x = radii * 100, y = min.norm[, 1], col = col.comp[1])
lines(x = radii * 100, y = min.norm[, 1], col = col.comp[1])
lines(x = radii[c(5, 7)] * 100, y = min.norm[c(5, 7), 1], col = col.comp[1], lty = "dashed")
lines(x = radii[c(9, 13)] * 100, y = min.norm[c(9, 13), 1], col = col.comp[1], lty = "dashed")
monsm.pat <- lambda2ratio(l0 = 10.3, l1 = 29.51, l2 = -621.9,
                          l3 = 2982, l4 = 42723)
lines(x = radii * 100, y = monsm.pat, col = col.comp[2])

# florencite-(Sm)
points(x = radii * 100, y = min.norm[, 2], col = col.comp[5])
lines(x = radii * 100, y = min.norm[, 2], col = col.comp[5])
lines(x = radii[c(7, 9)] * 100, y = min.norm[c(7, 9), 2], col = col.comp[5], lty = "dashed")
flosm.pat <- lambda2ratio(l0 = 9.17, l1 = 36.18, l2 = -656.9,
                          l3 = -1246, l4 = 63336)
lines(x = radii * 100, y = flosm.pat, col = col.comp[6])

box()

legend(x = 116, y = 8000000, title = "monazite-(Sm)",
       legend = c("data", "fit"),
       col = c(col.comp[1], col.comp[2]),
       lty = "solid", pch = c(1, NA),
       bty = "n", cex = 0.8)

legend(x = 106, y = 8000000, title = "florencite-(Sm)",
       legend = c("data", "fit"),
       col = c(col.comp[5], col.comp[6]),
       lty = "solid", pch = c(1, NA),
       bty = "n", cex = 0.8)

text(x = radii * 100, y = 80, labels = names(radii),
     cex = 0.8, font = 1, srt = 90, adj = 0)
axis(1, at = radii * 100, labels = NA,
     lwd = 0, lwd.ticks = 1,
     tcl = 0.25, mgp = c(-3, -1.5, 0),
     cex.axis = 0.5)
axis(1, lwd = 0, lwd.ticks = 1, tcl = -0.25,
     mgp = c(3, 0.3, 0))


# lambdas

par(mar = c(1, 0, 0.2, 0.2),
    mgp = c(0.1, 0, 0),
    xpd = TRUE)

plot(1, type = "n", bty = "n", xaxt = "n", yaxt = "n",
     xlim = c(-750, 750), ylim = c(-6500, 6500),
     xaxs = "i", yaxs = "i",
     xlab = expression(lambda[2] == "[-750, 750]"))

# florencite-(Sm)
lines(x = c(-750, -663, -750), y = c(5137, 5097, 3653), col = col.comp[6])
# monazite-(Sm)
lines(x = c(-750, -725, -750), y = c(3459, 3864, 3869), col = col.comp[2])
# samarskite-(Yb)
lines(x = c(714, 750), y = c(-6500, -6122), col = col.comp[4])
lines(x = c(-322, -361, -110, 255), y = c(-6500, -4971, 930, 6500), col = col.comp[4])
# xenotime-(Yb)
lines(x = c(79, 750), y = c(-6500, 873), col = col.comp[8])
lines(x = c(-750, -311, -192), y = c(-5499, 4624, 6500), col = col.comp[8])
# keiviite-(Yb)
lines(x = c(662, 750), y = c(-6500, -5547), col = col.comp[10])
lines(x = c(-739, -635, 56), y = c(-6500, -4104, 6500), col = col.comp[10])
# hingganite-(Yb)
lines(x = c(400, 750), y = c(-6500, -2686), col = col.comp[12])
lines(x = c(-555, -535, -176, 47), y = c(-6500, -5270, 3068, 6500), col = col.comp[12])

# these values from oneill.xlsx
points(x = c(-621, -656.9, -26.4, 128.3, 277.3, 39.6),
       y = c(2982, -1246, -939, 4538, 367, 683),
       col = c(col.comp[2], col.comp[6], col.comp[4],
               col.comp[8], col.comp[10], col.comp[12]),
       pch = 16)

text(x = c(-500, 450), y = c(4500, 3500), labels = c("Sm", "Yb"), col = "dimgrey")

arrows(x0 = c(-580, 400, 500), x1 = c(-700, 270, 650),
       y0 = c(4400, 3900, 3100), y1 = c(4050, 5000, 1500),
       length = 0.05, col = "dimgrey")

mtext(expression(lambda[3] == "[-6500, 6500]"), side = 2)

box()

mtext("b.", side = 3, line = -1, adj = -0.08)

dev.off()


# supplementary boundaries for fig 7 --------------------------------------

# ranges of the lambda values
l1.seq <- c(29.59, 36.18, -29.56, -72.26, -68.24, -48.11)
l2.seq <- seq(-750, 750, 4 * red.fact)
l3.seq <- seq(-6500, 6500, 40 * red.fact)
l4.seq <- c(46053, 63336, -11996, 43305, -37019, 16993)

# File name to save the data to hard drive
fn <- "patterndata.comp"

# Structure for the following data.frames
df.str <- data.frame(l1 = NA,
                     l2 = NA,
                     l3 = NA,
                     l4 = NA,
                     ree.max = NA, ree.max2 = NA,
                     eu.ratio = NA, eu.ratio2 = NA)

if (file.exists(fn)){
  load(fn)
} else
{
  # Initialise data.frame
  df <- df.str
  
  # Using intermediate data.frames for increased performance via df.str
  for (i in 1:6) {
    df.j <- df.str
    for(j in l2.seq){
      df.k <- df.str
      for (k in l3.seq){
        # generate a pattern
        ree.pat <- lambda2ratio(7, l1.seq[i], j, k, l4.seq[i],
                                Ce = 1, Eu = 1)
        # calculate the ppm values
        ree.ppm <- ree.pat * CI
        # add Y with Y/Ho = 25.5
        ree.ppm <- c(ree.ppm, ree.ppm[10] * 25.5)
        # calculate atomic values, incl Y mass
        ree.mol <- ree.ppm / c(mass, 88.90584)
        # find the maximum
        ree.max <- which.max(ree.mol)
        # zero Ce and Y
        ree.mol2 <- ree.mol
        ree.mol2[c(2,15)] <- 0
        ree.max2 <- which.max(ree.mol2)
        # Eu anomaly needed to make Eu mineral
        eu.ratio <- ree.mol[ree.max] / ree.mol[6]
        eu.ratio2 <- ree.mol2[ree.max2] / ree.mol2[6]
        
        df.k <- rbind(df.k, c(l1.seq[i], j, k, l4.seq[i],
                              ree.max, ree.max2,
                              eu.ratio, eu.ratio2))
      }
      print(j)
      # [-1,] to remove the first NA value formed using data.frame()
      df.j <- rbind(df.j, df.k[-1, ])
    }
    print(i)
    df <- rbind(df, df.j[-1, ])[-1, ]
  }
  save(df, file = fn)
}


pdf("comparison.pdf", width = fig.size / 2.54, height = fig.size / 2.54 / 5 * 2 * 2,
    pointsize = 10, family = "Bookman")

par(mfrow = c(2,3),
    mar = c(0, 0, 1, 0),
    oma = c(1, 1, 0.6, 1),
    cex = 1, xpd = TRUE)

for (i in unique(df$l1)){
  df.s <- df[df$l1 == i, ]
  df.m <- xtabs(ree.max2 ~ l2 + l3, data = df.s)
  image(df.m, col = hcl.colors(15, palette = "Plasma"), zlim = c(1, 15),
        xaxt = "n", yaxt = "n", useRaster = TRUE, bty = "n")
  box()
  mtext(text = bquote(lambda[1] == .(i) ~ ~ lambda[4] == .(l4.seq[match(i, unique(df$l1))])),
        side = 3, line = -0.1)
}

mtext(expression(lambda[2] == "[-750, 750]"), side = 1, outer = TRUE, line = 0.1)
mtext(expression(lambda[3] == "[-6500, 6500]"), side = 2, outer = TRUE, line = 0)

dev.off()


# pie key -----------------------------------------------------------------

pie(rep(1, 15), col = hcl.colors(15, palette = "Plasma"), labels = c(names(CI), "Y"))
